import os

import oss2
import requests
from src import common, config, project, log
myLogger = log.Logger()


def download(s=None, t=None):
    common.checkToken()

    if s is None:
        myLogger.error_logger('Please enter the correct file or directory')

    if t is None:
        myLogger.error_logger('Please enter the correct directory')

    if not t.endswith("/"):
        myLogger.error_logger('Please enter the correct directory')

    if s.endswith("/"):
        downloadDir(s, t)
    else:
        downloadFile(s, t)

def downloadDir(s=None, t=None):
    print("Directory download: "+s)
    data = {
        "dirPath": s,
        "projectId": project.getProjectId()
    }

    data = common.get(url=common.get_cloud_base_url() + '/file/getFilesByDirPath',params=data)

    for index in range(len(data)):
        if(data[index]['type'] == 0):
            temp_relative_path = t +data[index]['relativePath']

            abs_path = os.path.abspath(temp_relative_path)

            if(not os.path.isdir(abs_path)):
                print("Directory does not exist, create directory, path:"+abs_path)
                os.makedirs(abs_path)

        elif(data[index]['type'] == 1):
            temp_path = data[index]['path']
            temp_name = data[index]['name']

            t_path = t+data[index]['relativePath']

            t_path = t_path[0: len(t_path) - len(temp_name)]
            abs_path = os.path.abspath(t_path)
            downloadFile(temp_path, abs_path)

def downloadFile(s=None, t=None):
    print('Download file: '+s)
    data = {
        "filePath": s,
        "projectId": project.getProjectId()
    }


    data = common.get(url=common.get_cloud_base_url() + '/file/downloadFile', params=data)

    oss_path = data['ossPath']
    file_name = data['name']
    oss_endpoint = data['ossEndpoint']
    oss_bucket_name = data['ossBucketName']
    download_file_path = t+'/'+file_name
    if os.path.isfile(download_file_path):
        print("There is already a file with the same name locally, skip downloading the file")
    else:
        def percentage(consumed_bytes, total_bytes):
            if total_bytes:
                rate = int(100 * (float(consumed_bytes) / float(total_bytes)))
                print('\rDownload progress {0}% '.format(rate), end='')

        auth = oss2.StsAuth(data['accessKeyId'], data['accessKeySecret'], data['securityToken'])
        bucket = oss2.Bucket(auth, oss_endpoint, oss_bucket_name)

        # 判断文件是否存在,存在跳过，提示用户

        bucket.get_object_to_file(config.oss_file_path + oss_path, download_file_path, progress_callback=percentage);



import os, json

template = """{
    "id": "",
    "name": "",
    "desc": "",
    "cmd": '''''',
    "imageName": "",
    "inputs": [
        {
            "label": "",
            "key": "",
            "name": ""
        }
    ],
    "outputs": [
        {
            "label": "",
            "key": "",
            "name": ""
        }
    ]
}"""


def initTool(name):

    str = template.replace("\"name\": \"\",", "\"name\": \"" + name.strip() + "\",");
    cur_path = os.getcwd()
    print(template)
    with open(cur_path + '/tool.py', 'w', encoding="utf-8") as f:
        f.write(str)
    os.mkdir(os.getcwd() + '/input')
    os.mkdir(os.getcwd() + '/output')



import json
import os
import re
import uuid
import docker
import ast

from src import common, config, project, log
myLogger = log.Logger()

def push(file):

    if file is None:
        is_exist = os.path.exists(os.getcwd() + '/tool.py')
        if is_exist == False:
            myLogger.error_logger('Please initialize the tool')
        with open(os.getcwd() + '/tool.py', 'r', encoding='utf-8') as f:
            str = f.read()
            config_json = ast.literal_eval(str)
            checkToolConfig(config_json)

        pushImage(config_json)

    if file is not None:
        file = file.strip()
        is_exist = os.path.exists(file)
        if is_exist == False:
            myLogger.error_logger('Please enter the correct tool.py')
        with open(file, 'r', encoding='utf-8') as f:
            str = f.read()
            config_json = ast.literal_eval(str)
            checkToolConfig(config_json)
        pushImage(config_json)
    print("push successful")


def pushImage(config_json):
    if config_json['id'] == '':
        imageName = config_json['imageName']
        client = docker.from_env(timeout=100000000)
        image = client.images.get(imageName)
        path = str(uuid.uuid1()).replace("-", "")

        image.tag(common.get_docker_registry() + '/' + path, tag='latest')
        auth = common.get(url=common.get_cloud_base_url() + '/tool/pass')
        for line in client.images.push(common.get_docker_registry() + '/' + path, auth_config={'username': auth['account'], 'password': auth['password']}, stream=True, decode=True):
            print(line)
            if 'error' in line and line['error'] != '':
                print(line['error'])
                client.images.remove(common.get_docker_registry() + '/' + path + ':latest')
                myLogger.error_logger('Tool upload failed')
            if 'progressDetail' in line and (line['progressDetail'] != '' and line['progressDetail'] != {} and 'current' in line['progressDetail'] and 'total' in line['progressDetail']):
                current = line['progressDetail']['current']
                total = line['progressDetail']['total']
                if current > total:
                    current = total
                progress_num = (current / total)*100
                progress = round(progress_num, 2)
                print('uploading: '+str(progress)+"%")
        try:
            common.post(url=common.get_cloud_base_url() + '/tool/create', data={
                'name': config_json['name'],
                'desc': config_json['desc'],
                'cmd': config_json['cmd'],
                'inputs': config_json['inputs'],
                'outputs': config_json['outputs'],
                'path': path,
                'projectId': project.getProjectId()
            })
        except Exception as e:
            print(e)
        client.images.remove(common.get_docker_registry() + '/' + path + ':latest')
    if config_json['id'] != '':
        imageName = config_json['imageName']
        client = docker.from_env()
        image = client.images.get(imageName)
        path = str(uuid.uuid1()).replace("-", "")

        image.tag(common.get_docker_registry() + '/' + path, tag='latest')
        auth = common.get(url=common.get_cloud_base_url() + '/tool/pass')
        for line in client.images.push(common.get_docker_registry() + '/' + path, auth_config={'username': auth['account'], 'password': auth['password']}, stream=True, decode=True):
            if 'error' in line and line['error'] != '':
                print(line['error'])
                client.images.remove(common.get_docker_registry() + '/' + path + ':latest')
                myLogger.error_logger('Tool upload failed')
            print(line)
        try:
            common.post(url=common.get_cloud_base_url() + '/tool/create/version', data={
                'name': config_json['name'],
                'desc': config_json['desc'],
                'cmd': config_json['cmd'],
                'inputs': config_json['inputs'],
                'outputs': config_json['outputs'],
                'path': path,
                'id': config_json['id'],
                'projectId': project.getProjectId()
            })
        except Exception as e:
            print(e)
        client.images.remove(common.get_docker_registry() + '/' + path + ':latest')


def checkToolConfig(toolConfig):
    if toolConfig['imageName'] == '':
        myLogger.error_logger('Please enter the correct image name')
    if toolConfig['imageName'] != '':
        client = docker.from_env()
        images = client.images
        images.get(toolConfig['imageName'])
    if toolConfig['cmd'] == '':
        myLogger.error_logger("Command cannot be empty")
    if toolConfig['id'] == '':
        toolName = toolConfig['name']
        response = common.get(url=common.get_cloud_base_url() + '/tool/name', params={"name": toolName, "projectId": project.getProjectId()})
        if response is not None:
            myLogger.error_logger("Tool already exists, please do not submit again")
    toolVersionCmdParamList = []
    if toolConfig['cmd'] != '':
        toolVersionCmdParamList = analysis(cmd=toolConfig['cmd'])

    have_in_dir = False
    have_out_dir = False
    for cmdParam in toolVersionCmdParamList:
        if cmdParam['type'] == 4:
            have_in_dir = True
        if cmdParam['type'] == 5:
            have_out_dir = True

    toolInputFileList = toolConfig['inputs']

    inLabelSet = set()
    inFileKeyList = set()
    inFileKeyLength = 0
    for inputFile in toolInputFileList:
        if inputFile['label'] == '':
            myLogger.error_logger('Label cannot be blank')
        inLabelSet.add(inputFile['label'])
        if inputFile['key'] != '':
            inFileKeyList.add(inputFile['key'])
            inFileKeyLength = inFileKeyLength + 1
        else:
            if not have_in_dir:
                myLogger.error_logger('CMD does not match the parameter')

    outLabelSet = set()
    outFileKeyList = set()
    outDirFileSet = set()
    outFileKeyLength = 0
    toolOutputFileList = toolConfig['outputs']
    if len(toolOutputFileList) == 0:
        myLogger.error_logger('outputs cannot be empty')
    for outFile in toolOutputFileList:
        if outFile['name'] == '' or outFile['name'] is None:
            myLogger.error_logger('The output file name cannot be blank')

        if outFile['label'] == '':
            myLogger.error_logger('Label cannot be blank')

        outLabelSet.add(outFile['label'])
        if outFile['key'] != '':
            if '*' is outFile['name']:
                myLogger.error_logger('Output command line arguments are not available *')
            outFileKeyList.add(outFile['key'])
            outFileKeyLength = outFileKeyLength + 1
        else:
            if not have_out_dir:
                myLogger.error_logger('CMD does not match the parameter')

        if outFile['key'] == '' and '*' in outFile['name']:
            if outFile['name'].startswith('*.'):
                out_file_str_list = outFile['name'].split('.')
                if len(out_file_str_list) == 2:
                    myLogger.error_logger('OutName error')
                else:
                    if out_file_str_list[1] == '':
                        myLogger.error_logger('OutName error')
                    else:
                        if outFile['name'] in outDirFileSet:
                            myLogger.error_logger('Output file name repeat')
                        else:
                            outDirFileSet.add(outFile['name'])


    if inFileKeyLength != len(inFileKeyList):
        myLogger.error_logger("The input file key cannot be repeated")
    if outFileKeyLength != len(outFileKeyList):
        myLogger.error_logger("The output file key cannot be repeated")

    toolVersionParamsInFileKeySet = set()
    for toolVersionCmdParam in toolVersionCmdParamList:
        if toolVersionCmdParam['type'] == 2:
            toolVersionParamsInFileKeySet.add(toolVersionCmdParam['paramKey'])

    if len(toolVersionParamsInFileKeySet) == 0 and inFileKeyLength > 0:
        myLogger.error_logger('input error')


    toolVersionParamsOutFileKeySet = set()
    for toolVersionCmdParam in toolVersionCmdParamList:
        if toolVersionCmdParam['type'] == 3:
            toolVersionParamsOutFileKeySet.add(toolVersionCmdParam['paramKey'])

    if len(toolVersionParamsOutFileKeySet) == 0 and outFileKeyLength > 0:
        myLogger.error_logger('output error')

    isInFileEqual = inFileKeyList.difference(toolVersionParamsInFileKeySet)
    if len(isInFileEqual) != 0:
        myLogger.error_logger('Input key and cmd key do not match')

    isOutFileEqual = outFileKeyList.difference(toolVersionParamsOutFileKeySet)
    if len(isOutFileEqual) != 0:
        myLogger.error_logger('Output key and cmd key do not match')

def analysis(cmd):
    response = common.get(url=common.get_cloud_base_url() + '/tool/analysis/cmd', params={"cmd": cmd})
    return response




msg = {
    "-3" : "send mail exception",
    "-2" : "Request parameter exception",
    "-1" : "Service Error",

    "1000" : "User already exists",
    "1001" : "This user does not exist",
    "1002" : "Incorrect account password",
    "1003" : "Internal error",
    "1004" : "Authorization failed",
    "1005" : "User is not activated",
    "1006" : "User activated",
    "1007" : "Incorrect activation code",
    "1008" : "Password format is composed of 8-126 alphanumeric special characters",
    "1009" : "User information does not exist",
    "1010" : "Same username",
    "1020" : "The user has been authenticated with real name, and it is not allowed to modify the name and ID number",
    "1021" : "wrong identification number",
    "1022" : "Name must be in Chinese",
    "1023" : "The phone number is the same as the old phone number",
    "1024" : "Phone number format is not correct",
    "1025" : "The alternate email format is incorrect",
    "1026" : "User status error",
    "1027" : "The two passwords entered are not consistent",
    "1028" : "During real-name authentication, you cannot submit again",
    "1029" : "The front photo of the ID card does not exist",
    "1030" : "Verification code has expired",
    "1031" : "Picture format can only be jpg, jpeg, png",
    "1032" : "Alternate mailbox and mailbox duplicate",
    "1033" : "The start time is less than the end time",
    "1034" : "sortDate and sortCost cannot be sorted at the same time",
    "1035" : "User wallet does not exist",
    "1036" : "User nickname already exists",
    "1037" : "User disabled",
    "1038" : "User enabled",
    "1039" : "Insufficient user balance",
    "1040" : "User nickname cannot be mailbox",
    "1041" : "The photo on the back of the ID card does not exist",

    "1100" : "Wrong recharge type",
    "1101" : "Order does not exist",

    "1300" : "STS authorization failed",
    "1301" : "Internal method verification failed",

    "2000" : "User already exists",
    "2001" : "Authorization failed",
    "2002" : "Failed to request SSO service",
    "2003" : "STS authorization failed",
    "2004" : "The user already exists registration failed",
    "2005" : "User does not exist",
    "2006" : "Forbidden error",
    "2007" : "Insufficient balance, unable to operate, please recharge",
    "2008" : "Deactivation failed, please exit all projects first",
    "2009" : "Internal method verification failed",

    "2100" : "Project name already exists",
    "2101" : "Project does not exist",
    "2102" : "User is already in the project",
    "2103" : "The role does not exist in the project",
    "2104" : "The user was not found in the project",
    "2105" : "Only the owner has the right to modify permissions",
    "2106" : "Cannot modify the permissions of the project owner",

    "2200" : "Directory does not exist",
    "2201" : "File does not exist",
    "2202" : "Duplicate file id",
    "2203" : "Duplicate file name",
    "2204" : "Operation file attribution error",
    "2205" : "File upload error",
    "2206" : "Cannot move to the original directory",
    "2207" : "The file name format is wrong, the file name needs a suffix",
    "2208" : "Cannot move to the output directory",
    "2209" : "Cannot create a folder in the output directory",
    "2210" : "Files in the output directory cannot be renamed",
    "2211" : "File name/directory name is invalid",
    "2212" : "A file with the same name already exists",
    "2213" : "A directory with the same name already exists",
    "2214" : "Cannot move to the store directory",
    "2215" : "Files in the store directory cannot be renamed",
    "2216" : "Cannot be copied to the output directory",
    "2217" : "Cannot be copied to the store directory",
    "2218" : "Cannot copy the store directory and the files in it",
    "2219" : "Cannot be copied to the same folder",
    "2220" : "The directory already has a file or folder with the same name",


    "2300" : "Command parameters cannot use wildcards",
    "2301" : "Input file key does not match",
    "2302" : "Output file key does not match",
    "2303" : "OutName error",
    "2304" : "Duplicate input key",
    "2305" : "Tool does not exist",
    "2306" : "Tool file cannot be empty",
    "2307" : "Tool label cannot be repeated",
    "2308" : "Incorrect tool name",
    "2309" : "Tool is missing output file",
    "2310" : "Tool already exists, do not create it again",
    "2311" : "You cannot delete a tool version with only one version",
    "2312" : "The tool version does not exist",
    "2313" : "Tool input file label cannot be repeated",
    "2314" : "The label of the tool output file cannot be repeated",
    "2315" : "Duplicate output key",
    "2316" : "cmd number type value is wrong",


    "2400" : "The flow with the same name already exists",
    "2401" : "The flow node does not exist",
    "2402" : "The flow does not exist",
    "2403" : "The flow is being referenced",
    "2404" : "The flow version is being referenced",
    "2405" : "The flow version does not exist",
    "2406" : "At least one final output file is required in the flow",
    "2407" : "Duplicate flow reference parent node",
    "2408" : "Flow reference error",
    "2409" : "Please set the final output file in the flow leaf node",
    "2410" : "Duplicate node names are not allowed in the flow",
    "2411" : "Flow loop error",
    "2412" : "There is a disconnection error in the flow",


    "2500" : "Duplicate node names are not allowed in the job",
    "2501" : "Please set the final output file in the job leaf node",
    "2502" : "Duplicate job reference parent node",
    "2503" : "Job reference error",
    "2504" : "Job loop error",
    "2505" : "There is a disconnection error in the job",
    "2506" : "Job does not exist",
    "2507" : "The job is running and cannot be deleted",
    "2508" : "Duplicate output folder",
    "2509" : "Wrong output folder path",
    "2510" : "Stop error, job succeeded",
    "2511" : "Stop error, job has failed",
    "2512" : "Stop error, job has stopped",


    "2600" : "Resource pack does not exist",
    "2601" : "Duplicate resource pack name",
    "2602" : "Duplicate resource pack payment method",
    "2603" : "Update resource package name error",
    "2604" : "Resource pack price error",
    "2605" : "Cannot subscribe to self-published mall packages",
    "2606" : "Tag already exists",
    "2607" : "Can't repeat likes",
    "2608" : "Cancel like error",
    "2609" : "Resource package deduction method does not exist",
    "2610" : "Resource pack already has a tag with the same name",
    "2611" : "This label does not exist in the resource pack",
    "2612" : "This operation cannot be performed on the resource pack label",

    "2621" : "No permission to upload the file",
    "2622" : "No permission to upload the tool",
    "2623" : "No permission to upload the flow",

    "2641" : "Duplicate upload file",
    "2642" : "Duplicate upload tool",
    "2643" : "Duplicate upload flow",

    "2661" : "The resource pack is already subscribed",
    "2662" : "The resource subscription package does not exist",
    "2663" : "Resource subscription package cannot be imported repeatedly",
    "2664" : "Resource subscription directory does not exist",
    "2665" : "The flow does not belong to the user",
    "2666" : "Cannot update this resource subscription package",

    "2681" : "Subscription package added to project duplicate error",
    "2682" : "No permission to import into the project",
    "2683" : "You cannot import the same subscription package repeatedly into the same project",
    "2684" : "Resource pack is not added to the project",
    "2685" : "Resource is locked, import cannot be canceled",
    "2686" : "This operation cannot be performed on imported resources",

    "2700" : "Message does not exist",
    "2701" : "The message cannot be repeated to set the read status",
    "2702" : "Non-deletable announcement",
    "2703" : "Can only delete own messages",

    "2800" : "The problem does not exist",
    "2801" : "No permission to delete this question",
    "2802" : "The problem does not exist",
    "2803" : "No permission to delete this label",

    "2900": "The final output file name is wrong",
    "2901": "Cmd format error",
    "2902": "Duplicate cmd key",
    "2903": "Operational resources are locked",
    "2904": "File key does not match cmd key",
    "2905": "Worker node type does not exist",
    "2906": "Oss file reference error",
    "2907": "Worker type not selected",
    "2908": "Wrong Worker type selection",
    "2909": "The number of references for this resource is zero",
    "2910": "Text conversion error",
    "2911": "Input file list error",
    "2912": "Output file list error",
    "2913": "CMD does not match the parameter"
  }


from src import common, config, project, log
import os, json, docker
myLogger = log.Logger()

template = {
    "id": "",
    "name": "",
    "desc": "",
    "cmd": '''''',
    "imageName": "",
    "inputs": [
        {
            "label": "",
            "key": "",
            "name": ""
        }
    ],
    "outputs": [
        {
            "label": "",
            "key": "",
            "name": ""
        }
    ]
}

def pull(toolVersionId):

    pullToolEntity = common.get(url=common.get_cloud_base_url() + '/tool/toolversion/id', params={'id': toolVersionId, 'projectId': project.getProjectId()})
    if pullToolEntity is None:
        myLogger.error_logger('The tool version id error')
    toolEntity = common.get(url=common.get_cloud_base_url() + '/tool/tool/id', params={'id': pullToolEntity['id'], 'projectId': project.getProjectId()})
    if toolEntity is None:
        myLogger.error_logger("Tool does not exist")

    template['name'] = toolEntity['name']
    template['desc'] = pullToolEntity['desc']
    template['id'] = toolEntity['id']
    auth = common.get(url=common.get_cloud_base_url() + '/tool/pass')
    client = docker.from_env(timeout=100000000)
    print("pulling, please wait")
    client.images.pull(repository=common.get_docker_registry() + '/' + pullToolEntity['path'],
                       tag="latest",
                       auth_config={'username': auth['account'], 'password': auth['password']})

    image = client.images.get(common.get_docker_registry() + '/' + pullToolEntity['path'] + ':latest')
    image.tag(repository=toolEntity['id'], tag=str(pullToolEntity['version']))
    client.images.remove(common.get_docker_registry() + '/' + pullToolEntity['path'] + ':latest')
    template['imageName'] = toolEntity['id'] + ':' + str(pullToolEntity['version'])
    template['inputs'] = pullToolEntity['inputs']
    template['outputs'] = pullToolEntity['outputs']
    template['version'] = pullToolEntity['version']

    tempStr = json.dumps(template, indent=4, ensure_ascii=False)
    tempStr = tempStr.replace("\"cmd\": \"\"", "\"cmd\": '''" + pullToolEntity['cmd'] + "'''")
    with open(os.getcwd() + '/tool.py', 'w', encoding='utf-8') as f:
        f.write(tempStr)
    os.mkdir(os.getcwd() + '/input')
    os.mkdir(os.getcwd() + '/output')
    print("pull successful")


import json
import os
import requests

from src import config, codeMsg

from src import log
myLogger = log.Logger()

def getCodeMsg(msgCode):
    return codeMsg.msg[msgCode]


def get_login_base_url(env=None):
    if env == None:
        env = getEnv()

    if env == 'develop':
        return config.login_base_url_develop
    elif env == 'stage':
        return config.login_base_url_stage
    elif env == 'master':
        return config.login_base_url_master


def get_cloud_base_url(env=None):
    if env == None:
        env = getEnv()
    if env == 'develop':
        return config.cloud_base_url_develop
    elif env == 'stage':
        return config.cloud_base_url_stage
    elif env == 'master':
        return config.cloud_base_url_master


def get_docker_registry(env=None):
    if env == None:
        env = getEnv()
    if env == 'develop':
        return config.docker_registry_develop
    elif env == 'stage':
        return config.docker_registry_stage
    elif env == 'master':
        return config.docker_registry_master



def getEnv():
    cli_path = config.USER_BASE_PATH + '/' + config.cli_dir
    config_path = cli_path + '/' + config.cli_json
    is_exist = os.path.exists(config_path)
    if not is_exist:
        myLogger.error_logger('Please login again')

    file = open(config_path, 'r')
    js = file.read()
    dic = json.loads(js)
    env = dic['env']
    if env == "":
        myLogger.error_logger("Please login again")
    file.close()

    return env

# -----------------授权相关 start-------------------------------------------------
def getToken():
    cli_path = config.USER_BASE_PATH + '/' + config.cli_dir
    config_path = cli_path + '/' + config.cli_json
    is_exist = os.path.exists(config_path)
    if not is_exist:
        myLogger.error_logger('Please login again')

    file = open(config_path, 'r')
    js = file.read()
    dic = json.loads(js)
    token = dic['access_token']
    if token == "":
        myLogger.error_logger("Please login again")
    file.close()

    return token

def checkToken(token=None, env=None):
    if token is not None:
        params = {'token': token}
        post(url=get_login_base_url(env) + '/user/check/token', data=params, haveToken=False)
    else:
        params = {'token': getToken()}
        post(url=get_login_base_url(env) + '/user/check/token', data=params, haveToken=False)


# -----------------授权相关 end-------------------------------------------------

# ----------------数据请求相关 end-------------------------------------------------

def get(url, params=None, haveToken=True):
    if haveToken == True:
        token = getToken()
        response = requests.get(url=url, params=params, headers={
            'Content-Type': 'application/json',
            'ACCESS_TOKEN': token
        })
        return dealResponse(response=response)
    if haveToken == False:
        response = requests.get(url=url, params=params, headers={
            'Content-Type': 'application/json',
        })
        return dealResponse(response=response)


def post(url, data, haveToken=True):
    if haveToken == True:
        token = getToken()
        response = requests.post(url=url, data=json.dumps(data), headers={
            'Content-Type': 'application/json',
            'ACCESS_TOKEN': token
        })
        return dealResponse(response=response)

    if haveToken == False:
        response = requests.post(url=url, data=json.dumps(data), headers={
            'Content-Type': 'application/json'
        })
        return dealResponse(response=response)

def dealResponse(response):
    if response.status_code == 200:
        return response.json()['data']
    if response.status_code == 401:
        myLogger.error_logger("Authorization failed, please login again")
        
    if response.status_code >= 400 and response.status_code < 500:
        resJson = response.json()
        value = getCodeMsg(str(resJson['msgCode']))
        myLogger.error_logger(value)

    if response.status_code >= 500:
        resJson = response.json()
        value = getCodeMsg(str(resJson['msgCode']))
        myLogger.error_logger(value)


import fire
from src import initTool, pull, download, common, upload, config, logout, run, login, push, project


class Cmd(object):
    """Command line tool\n
    Mainly used for flow hub platform file and tool transfer. If you encounter any problems during use, please report related problems to flowhub@163.com, and we will reply to your message as soon as possible.
    login:
        The user login, the user needs to complete the login first before performing other operations of fkit
        --k key 
        Need to get password from webpage personal center
        E.g:fkit login --k=a18607940dd611eb9d6e024255358b77

    logout:
        The user logout. After completing the fkit tool operation, perform this operation to exit to prevent others from embezzling and fraudulent use
        E.g:fkit logout

    upload:
        Upload files or folders to the platform account
        --s source 
        The path of the file or folder to be uploaded under the current computer or cluster
        --t target 
        Upload to the directory path under the platform account
        E.g:fkit upload --s=./data.txt --t=/data/

    download:
        File or folder download
        --s source 
        Files or folders that need to be downloaded from the platform
        --t target 
        Download to the specified directory
        E.g:fkit download --s /output/venn/data.txt --t ./

    push:
        Upload the tool to the platform account
        --f 
        Optionally specify the location of tool.py, if not set, look for the tool.py file in the current folder
        E.g:fkit push --f=./tool.py

    pull:
        Pull online mirror, pull tool mirror from platform to local further development
        --id 
        Get the ID of the tool version from the web
        E.g:fkit pull --id=263347730d3311ebb63c024255358b77

    initTool:
        The initialization tool will generate the tool template tool.py file in the current directory. The user needs to configure the tool.py file according to the local development tool image
        E.g:fkit initTool demo

    run:
        Test the locally developed tool image and upload it to the platform after normal operation. Find the tool.py file in the current folder by default, please run it in the tool folder
        E.g:fkit run

    project:
        List the user's project, and can switch the current project
        
    version:
        View the version of the fkit command line tool
        E.g:fkit version
    """
    def login(self, k=None, env=None):
        """

        """
        login.login(k=k, env=env)
    def logout(self):
        """

        """
        logout.logout()
    def upload(self, s=None, t=None):
        """

        """
        common.checkToken()
        project.checkProject()
        upload.upload(s=s, t=t)
    def download(self, s=None, t=None):
        """

        """
        common.checkToken()
        project.checkProject()
        download.download(s=s, t=t)
    def push(self, f=None):
        """

        """
        common.checkToken()
        project.checkProject()
        push.push(file=f)
    def pull(self, id):
        """

        """
        common.checkToken()
        project.checkProject()
        pull.pull(toolVersionId=id)
    def initTool(self, name):
        """

        """
        common.checkToken()
        initTool.initTool(name)

    def run(self):
        """

        """
        common.checkToken()
        run.run()
    def project(self):
        common.checkToken()
        project.view()

    def version(self):
        """

        """
        return config.version

def init():
    fire.Fire(Cmd)


import logging  # 引入logging模块
import os.path
import time
import traceback
import sys
from src import config

class Logger:
    def __init__(self):
        # 第一步，创建一个logger
        logger = logging.getLogger()
        logger.setLevel(logging.INFO)  # Log等级总开关
        
        # 第二步，创建一个handler，用于写入日志文件
        cli_path = config.USER_BASE_PATH + '/' + config.cli_dir
        log_path = cli_path + '/Logs/'
        is_exist = os.path.exists(log_path)
        if not is_exist:
            os.makedirs(log_path)
        log_name = log_path + "error" + '.log'
        logfile = log_name
        fh = logging.FileHandler(logfile, encoding="utf-8", mode='w')
        fh.setLevel(logging.ERROR)  # 输出到file的log等级的开关
        # 第三步，定义handler的输出格式
        formatter = logging.Formatter("%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s: %(message)s")
        fh.setFormatter(formatter)
        
        # 再创建一个handler，用于输出到控制台    
        ch = logging.StreamHandler()  
        ch.setLevel(logging.CRITICAL)
        
        # 第四步，将logger添加到handler里面
        logger.addHandler(fh)
        logger.addHandler(ch)

    @staticmethod
    def error_logger(content):
        string = "\n    "
        for line in traceback.format_stack():
            string += line.strip()

        string += "\n    "+content
        logging.error(string)
        print("ERROR:" + content)
        sys.exit()

    @staticmethod
    def critical_logger(content):
        string = "\n    "
        for line in traceback.format_stack():
            string += line.strip()

        string += "\n   "+content
        logging.critical(string)
        sys.exit()



import setuptools
from src import config

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fkit",
    version=config.version,
    author='fkit group',
    author_email='flowhub@yeah.net',
    description='ft command line tool',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://gitee.com/tentech/cli.git',
    packages=setuptools.find_packages(),
    install_requires=[
        'docker == 4.2.2',
        'fire == 0.3.1',
        'requests == 2.24.0',
        'oss2 == 2.12.1',
        'pip >= 20.3.4'
    ],
    include_package_data=True,  # 自动打包文件夹内所有数据
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    platforms='any',
    entry_points={
        'console_scripts': [
            'fkit=src.cli:main'
        ]
    }
)



import json
import os

import oss2
import requests
from oss2 import SizedFileAdapter, determine_part_size
from oss2.models import PartInfo
import hashlib
import base64
import zlib

from src import config, project, common, log
myLogger = log.Logger()

def get_file_md5(file_name):
    """
    计算文件的md5
    :param file_name:
    :return:
    """
    m = hashlib.md5()   #创建md5对象
    with open(file_name,'rb') as fobj:
        while True:
            data = fobj.read(4096)
            if not data:
                break
            m.update(data)  #更新md5对象

    return m.hexdigest()    #返回md5对象

def get_file_sha1(fineName):
    with open(fineName, 'rb') as f:
        sha1 = hashlib.sha1()
        while True:
            data = f.read(4096)
            if not data:
                break
            sha1.update(data)
    return sha1.hexdigest()


def get_file_crc32(file_path):
    prev = 0
    for eachLine in open(file_path, "rb"):
        prev = zlib.crc32(eachLine, prev)
    return str("%X" % (prev & 0xFFFFFFFF))

def upload(s=None, t=None):

    checkToken()
    if s is None:
        myLogger.error_logger('Please enter the correct file or directory')

    if t is None:
        myLogger.error_logger('Please enter the correct target upload directory')

    if not t.endswith('/'):
        myLogger.error_logger('Please enter the correct target upload directory')

    is_upload_dir = False
    is_upload_dir_file = False
    is_upload_file = False
    if s.endswith('/*') :
        t_s = s[:-1]
        is_dir = os.path.isdir(t_s)
        if is_dir:
            is_upload_dir_file = True
            # 检测目录或文件是否重复
            dir_files = os.listdir(t_s)  # 得到该文件夹下所有的文件
            for file in dir_files:
                file_path = os.path.join(t_s, file)  # 路径拼接成绝对路径
                if os.path.isfile(file_path):  # 如果是文件，就打印这个文件路径
                    temp_file_path = t + file_path.replace(t_s, "")

                    check_dir_or_file_exist(temp_file_path)
                if os.path.isdir(file_path):  # 如果目录，就递归子目录
                    temp_dir_path = t + file_path.replace(t_s, "") + '/'

                    check_dir_or_file_exist(temp_dir_path)
            uploadDir(t_s, t)
    elif s.endswith('/') :
        is_dir2 = os.path.isdir(s)
        if is_dir2 :
            is_upload_dir = True
            abs_path = os.path.abspath(s)
            print(os.path.split(abs_path))
            split_list = os.path.split(abs_path)
            last_dir_name = split_list[len(split_list) - 1]

            file_dir = t + last_dir_name+'/'
            check_dir_or_file_exist(file_dir)
            create_dir(last_dir_name, t)
            uploadDir(s, file_dir)
    else:
        is_file = os.path.isfile(s)
        if is_file:
            is_upload_file = True
            p, f = os.path.split(s)
            temp_path = t + f
            print("Check for duplicate files:" + temp_path)

            check_dir_or_file_exist(temp_path)
            uploadFile(s, t)

    if not is_upload_file and not is_upload_dir and not is_upload_dir_file:
        myLogger.error_logger('Please enter the correct file or directory')


def getToken():
    cli_path = config.USER_BASE_PATH + '/' + config.cli_dir
    config_path = cli_path + '/' + config.cli_json
    is_exist = os.path.exists(config_path)
    if not is_exist:
        myLogger.error_logger('Please login again')

    file = open(config_path, 'r')
    js = file.read()
    dic = json.loads(js)
    token = dic['access_token']
    file.close()

    return token

def uploadDir(dir, target_path):
    print("Upload folder:"+dir+",   Target path:"+target_path)

    for root, dirs, files in os.walk(dir):
        relative_path = root.replace(dir, "")
        path = target_path[:-1] + relative_path+'/'

        if not root.endswith('/'):
            root = root + '/'

        for temp_dir in dirs:
            create_dir(temp_dir, path)

        for file in files:
            abs_path = root + file

            temp_path = target_path + os.path.join(root, "")[len(dir):]
            uploadFile(abs_path, temp_path)

def check_dir_or_file_exist(dir_path):

    data = {
        "dirPath": dir_path,
        "projectId": project.getProjectId()
    }

    r = common.post(url=common.get_cloud_base_url() + '/file/checkFileOrDirExist', data=data)

def create_dir(dir_name, dir_path):

    data = {
        "name": dir_name,
        "dirPath": dir_path,
        "projectId": project.getProjectId()
    }

    r = common.post(url=common.get_cloud_base_url() + '/file/createDir',  data=data)


upload_total_size = 0
upload_uploaded_size = 0
def percentage(consumed_bytes, total_bytes):
    """进度条回调函数，计算当前完成的百分比

    :param consumed_bytes: 已经上传/下载的数据量
    :param total_bytes: 总数据量
    """
    if total_bytes:
        rate = 0
        if upload_total_size > 0:
            consumed_bytes = upload_uploaded_size + consumed_bytes
            rate = int(100 * (float(consumed_bytes) / float(upload_total_size)))
        if upload_total_size == 0:
            rate = 100
        print('\r{0}% '.format(rate), end='')

def uploadFile(file, target_path):
    print("Upload file: " + file)
    global upload_total_size
    upload_total_size = 0
    global upload_uploaded_size
    upload_uploaded_size = 0
    file_name = os.path.basename(file)
    file_path = os.path.abspath(file)
    total_size = os.path.getsize(file_path)

    md5_value = get_file_md5(file_path)
    sha1_value = get_file_sha1(file_path)
    crc32_value = get_file_crc32(file_path)
    print(md5_value)
    print(sha1_value)
    print(crc32_value)
    body = {
        "name": file_name,
        "type": 1,
        "dirPath": target_path,
        "projectId": project.getProjectId(),
        "md5": md5_value,
        "sha1": sha1_value,
        "crc32": crc32_value
    }

    data = common.post(url=common.get_cloud_base_url() + '/file/uploadFile', data=body)

    file_id = data['fileId']
    oss_path = data['ossPath']
    oss_endpoint = data['ossEndpoint']
    oss_bucket_name = data['ossBucketName']
    key = 'file/'+oss_path
    passUpload = data["passUpload"]

    if passUpload == 0:
        auth = oss2.StsAuth(data['accessKeyId'], data['accessKeySecret'], data['securityToken'])
        bucket = oss2.Bucket(auth, oss_endpoint, oss_bucket_name)

        # print("CPU的核数为：{}".format(cpu_count()))
        # determine_part_size方法用于确定分片大小。

        # 20M 一块
        if total_size < 1024:
            print("File size: "+str(int(total_size)) +"B")
        elif total_size <= 1024 * 1024:
            print("File size: "+str(int(total_size/1024)) +"KB")
        elif total_size <= 1024 * 1024 * 1024:
            print("File size: "+str(int(total_size/1024/1024)) +"MB")
        else:
            print("File size: " + str(int(total_size / 1024 / 1024 / 1024)) + "GB")

        upload_total_size = total_size
        if(total_size > 1024*1024*20):
            part_size = determine_part_size(total_size, preferred_size=(1024*1024*20))

            # 初始化分片。
            # 如需在初始化分片时设置文件存储类型，请在init_multipart_upload中设置相关headers，参考如下。
            # headers = dict()
            # headers["x-oss-storage-class"] = "Standard"
            # upload_id = bucket.init_multipart_upload(key, headers=headers).upload_id
            upload_id = bucket.init_multipart_upload(key).upload_id
            parts = []

            with open(file_path, 'rb') as fileobj:

                # 逐个上传分片。
                part_number = 1
                offset = 0
                while offset < total_size:
                    num_to_upload = min(part_size, total_size - offset)
                    # 调用SizedFileAdapter(fileobj, size)方法会生成一个新的文件对象，重新计算起始追加位置。
                    result = bucket.upload_part(key, upload_id, part_number,
                                                SizedFileAdapter(fileobj, num_to_upload), progress_callback=percentage)
                    parts.append(PartInfo(part_number, result.etag))

                    offset += num_to_upload
                    upload_uploaded_size = offset
                    part_number += 1
            # 完成分片上传。
            # 如需在完成分片上传时设置文件访问权限ACL，请在complete_multipart_upload函数中设置相关headers，参考如下。
            # headers = dict()
            # headers["x-oss-object-acl"] = oss2.OBJECT_ACL_PRIVATE
            # bucket.complete_multipart_upload(key, upload_id, parts, headers=headers)

            bucket.complete_multipart_upload(key, upload_id, parts)

            # 验证分片上传。
            with open(file_path, 'rb') as fileobj:
                assert bucket.get_object(key).read() == fileobj.read()

        else:
            bucket.put_object_from_file(key, file_path, progress_callback=percentage)

    needInsertVerify = 1
    if(passUpload == 1) :
        needInsertVerify = 0
    # 上传完毕后更新文件状态
    params = {
        'fileId': file_id,
        'status': 1,
        'projectId': project.getProjectId(),
        'needInsertVerify': needInsertVerify,
        'md5': md5_value,
        'sha1': sha1_value,
        'crc32': crc32_value
    }

    data = common.post(url=common.get_cloud_base_url() + '/file/uploadFileFinish', data=params)
    print(data)
    if(data['status'] == 1):
        if passUpload == 1:
            print(file_path + "   Quick upload complete")
        else :
            print(file_path + "   Upload complete")
    elif data['status'] == 0:
        print(file_path+"   Upload failed. There is a file with the same name in the same directory")


def checkToken():
    params = {'token': getToken()}

    r = common.post(url=common.get_login_base_url() + '/user/check/token', data=params)



import os

version = '0.0.28'

# 开发
login_base_url_develop = 'http://develop.flowhub.com.cn:8080'
cloud_base_url_develop = 'http://develop.flowhub.com.cn:8081'
docker_registry_develop = 'develop.flowhub.com.cn:5000'

# 测试
login_base_url_stage = 'https://stage.flowhub.com.cn/backend'
cloud_base_url_stage = 'https://stage.flowhub.com.cn/cloud'
docker_registry_stage = 'stage.flowhub.com.cn:5000'

# 生产
login_base_url_master = 'https://www.flowhub.com.cn/backend'
cloud_base_url_master = 'https://www.flowhub.com.cn/cloud'
docker_registry_master = 'www.flowhub.com.cn:5000'

cli_dir = '.cli'
USER_BASE_PATH = os.path.expanduser('~')
cli_json = '.cli.json'

oss_file_path = 'file/'



import os
from src import config


def logout():
    cli_path = config.USER_BASE_PATH + '/' + config.cli_dir + '/' + config.cli_json
    is_exist = os.path.exists(cli_path)
    if is_exist:
        os.remove(cli_path)
    print("Logout successfully")


import docker, os, json, re
from src import push, log
myLogger = log.Logger()

def run():

    configJsonExist = os.path.exists(os.getcwd() + '/tool.py')
    if not configJsonExist:
        myLogger.error_logger("The tool.py file cannot be found, please make sure to execute the command in the correct path")

    inputExist = os.path.exists(os.getcwd() + '/input')
    outputExist = os.path.exists(os.getcwd() + '/output')
    if not inputExist:
        myLogger.error_logger("Missing input folder, please create \"input\" folder in execution path")
    if not outputExist:
        myLogger.error_logger("Missing output folder, please create \"output\" folder in execution path")
    with open(os.getcwd() + '/tool.py', 'r', encoding='utf-8') as f:
        str = f.read()
        str = str.replace("'''", "\"")
        str = str.replace("\r","").replace("\n","")
        config_json = json.loads(str)

        push.checkToolConfig(config_json)
        cmdParams = push.analysis(config_json['cmd'])
        taskCmdParamEntityList = cmdParams

    taskCmd = config_json['cmd']
    inputs = []
    for input in config_json['inputs']:
        input['type'] = 0
        inputs.append(input)
    outputs = []
    for output in config_json['outputs']:
        output['type'] = 1
        outputs.append(output)
    taskFileEntityList = inputs + outputs
    pattern = '\#\{.*?\}'
    match_list = re.findall(pattern=pattern, string=taskCmd)
    for i in range(len(match_list)):
        exist = False
        for taskCmdParamEntity in taskCmdParamEntityList:
            if taskCmdParamEntity['index'] == i:
                exist = True

                if taskCmdParamEntity['type'] == 2 or taskCmdParamEntity['type'] == 3:
                    for taskFileEntity in taskFileEntityList:
                        if taskFileEntity['key'] != '' and '*' in taskFileEntity['name']:
                            myLogger.error_logger('Please fill in the exact command line input file name during the test run')

                        if taskFileEntity['key'] == taskCmdParamEntity['paramKey']:
                            if taskFileEntity['type'] == 0:
                                taskCmd = taskCmd.replace(match_list[i], taskCmdParamEntity['prefix'] + "/input/" + taskFileEntity['name'])
                            if taskFileEntity['type'] == 1:
                                taskCmd = taskCmd.replace(match_list[i], taskCmdParamEntity['prefix'] + "/output/" + taskFileEntity['name'])
                if taskCmdParamEntity['type'] == 0 or taskCmdParamEntity['type'] == 1:
                    taskCmd = taskCmd.replace(match_list[i], taskCmdParamEntity['prefix'] + taskCmdParamEntity['paramValue'])

                if taskCmdParamEntity['type'] == 4:
                    taskCmd = taskCmd.replace(match_list[i], taskCmdParamEntity['prefix'] + '/input/')

                if taskCmdParamEntity['type'] == 5:
                    taskCmd = taskCmd.replace(match_list[i], taskCmdParamEntity['prefix'] + '/output/')

        if exist == False:
            taskCmd = taskCmd.replace(match_list[i], '')

    runDocker(taskCmd=taskCmd, imageName=config_json['imageName'])
    checkOutputFile(outputs)
    print('run successful')

def getAllFiles(path):
    paths = []
    for root, dirs, files in os.walk(path):
        for file in files:
            paths.append({'dir': root, 'file': file})

    return paths

def checkOutputFile(outputFiles):
    paths = getAllFiles(os.getcwd() + '/output/')
    allFindFiles = []
    for file in outputFiles:
        if file['name'].startswith('*') == False:
            findFiles = []
            for filePath in paths:
                if filePath['file'] == file['name']:
                    findFiles.append(filePath)
                    allFindFiles.append(filePath)
            if len(findFiles) == 0:
                myLogger.error_logger('Missing output file')
            if len(findFiles) > 1:
                myLogger.error_logger('Output duplicate name file')

    for file in outputFiles:
        if file['name'].startswith('*'):
            suffix = os.path.splitext(file['name'])[1]
            findFiles = []
            for filePath in paths:
                if os.path.splitext(filePath['file'])[1] == suffix:
                    flag = 0
                    for tempFile in allFindFiles:
                        if filePath['file'] == tempFile['file']:
                            flag = 1
                    if flag == 0:
                        findFiles.append(filePath)
                        allFindFiles.append(filePath)

            if len(findFiles) == 0:
                myLogger.error_logger('Not matched to general configuration file')
            if len(findFiles) > 1:
                myLogger.error_logger('Matching to multiple pass through files')

def runDocker(taskCmd, imageName):
    client = docker.from_env()
    container = client.containers.run(image=imageName, auto_remove=True, detach=True, stdout=True, stderr=True, command=taskCmd,
                          volumes={
                              os.getcwd() + '/input/': {'bind': '/input', 'mode': 'rw'},
                              os.getcwd() + '/output/': {'bind': '/output', 'mode': 'rw'}
                          },
                          working_dir='/'
                          )
    for line in container.logs(stream=True):
        print(line)



from src import config, upload, common
import requests, os, json


def view():
    project_list = common.get(common.get_cloud_base_url() + '/project/list?projectName=')

    for index, project in enumerate(project_list):
        print('Project id：' + str(index), '    Project name：' + project['name'])

    print('-----------Please select project id--------------')

    project_id = input()
    project = project_list[int(project_id)]

    cli_path = config.USER_BASE_PATH + '/' + config.cli_dir
    config_path = cli_path + '/' + config.cli_json
    config_json = {}
    with open(config_path, 'r', encoding='utf-8') as f:
        config_json = json.load(f)
    config_json['project_id'] = project['id']

    with open(config_path, 'w', encoding='utf-8') as f:
        json.dump(config_json, f, indent=2, sort_keys=True, ensure_ascii=False)

def getProjectId():
    cli_path = config.USER_BASE_PATH + '/' + config.cli_dir
    config_path = cli_path + '/' + config.cli_json
    config_json = {}
    with open(config_path, 'r', encoding='utf-8') as f:
        config_json = json.load(f)
    return config_json['project_id']

def checkProject():
    cli_path = config.USER_BASE_PATH + '/' + config.cli_dir
    config_path = cli_path + '/' + config.cli_json
    with open(config_path, 'r', encoding='utf-8') as f:
        config_json = json.load(f)

    if ('project_id' in config_json.keys()) is False:
        view()

    if ('project_id' in config_json.keys()) is True:
        cli_path = config.USER_BASE_PATH + '/' + config.cli_dir
        config_path = cli_path + '/' + config.cli_json
        config_json = {}
        with open(config_path, 'r', encoding='utf-8') as f:
            config_json = json.load(f)

        try:
            response = common.get(common.get_cloud_base_url() + '/project/info',
                                params={'projectId': config_json['project_id']})
            print('Current project:' + response['name'])
        except Exception as e:
            view()




import json
from src import common, config, project, log
import getpass, os
myLogger = log.Logger()

def login(k, env):
    cli_path = config.USER_BASE_PATH + '/' + config.cli_dir
    config_path = cli_path + '/' + config.cli_json
    config_json = {}
    if env == None:
        env = 'master'
    else:
        if env != 'develop' and env != 'stage' and env != 'master':
            myLogger.error_logger("error env")

    if k is None:
        print("Please enter your email address")
        account = input()
        password = getpass.getpass("Please enter your password")

        data = {
            "email": account,
            "password": password
        }
        response = common.post(url=common.get_login_base_url(env) + '/user/login', haveToken=False, data=data)
        token = response['token']
        cli_path = config.USER_BASE_PATH + '/' + config.cli_dir

        print(cli_path)
        is_exist = os.path.exists(cli_path)
        if not is_exist:
            os.makedirs(cli_path)

        config_path = cli_path + '/' + config.cli_json
        json_exist = os.path.exists(config_path)
        if json_exist:
           os.remove(config_path)

        config_json['access_token'] = token
        config_json['env'] =env
        with open(config_path, 'w', encoding='utf-8') as f:
            json.dump(config_json, f, indent=2, sort_keys=True, ensure_ascii=False)

        print("login successful")
        project.checkProject()

    if k is not None:
        common.checkToken(token=k, env=env)
        token = k
        cli_path = config.USER_BASE_PATH + '/' + config.cli_dir
        is_exist = os.path.exists(cli_path)
        if not is_exist:
            os.makedirs(cli_path)

        config_path = cli_path + '/' + config.cli_json
        json_exist = os.path.exists(config_path)
        if json_exist:
            os.remove(config_path)
        config_json['access_token'] = token
        config_json['env'] = env
        with open(config_path, 'w', encoding='utf-8') as f:
            json.dump(config_json, f, indent=2, sort_keys=True, ensure_ascii=False)

        print("login successful")
        project.checkProject()
